@@ -1,44 +1,28 @@
-#![warn(
-    clippy::semicolon_if_nothing_returned,
-    // New clippy lints, not yet stable
-    // clippy::std_instead_of_core,
-    // clippy::std_instead_of_alloc,
-    // clippy::alloc_instead_of_core,
-    missing_docs,
-    rust_2018_idioms,
-    rustdoc::missing_crate_level_docs,
-    trivial_casts,
-    trivial_numeric_casts,
-    unused_extern_crates,
-    unused_import_braces,
-    unused_qualifications,
-    variant_size_differences
-)]
-#![doc(test(attr(forbid(unsafe_code))))]
-#![doc(test(attr(deny(
-    missing_debug_implementations,
-    trivial_casts,
-    trivial_numeric_casts,
-    unused_extern_crates,
-    unused_import_braces,
-    unused_qualifications,
-))))]
-#![doc(test(attr(warn(rust_2018_idioms))))]
+#![doc(test(attr(
+    allow(
+        unknown_lints,
+        // Problematic handling for foreign From<T> impls in tests
+        // https://github.com/rust-lang/rust/issues/121621
+        non_local_definitions,
+        // Some tests use foo as name
+        clippy::disallowed_names,
+    ),
+    deny(
+        missing_debug_implementations,
+        rust_2018_idioms,
+        trivial_casts,
+        trivial_numeric_casts,
+        unused_extern_crates,
+        unused_import_braces,
+        unused_qualifications,
+        warnings,
+    ),
+    forbid(unsafe_code),
+)))]
 // Not needed for 2018 edition and conflicts with `rust_2018_idioms`
 #![doc(test(no_crate_inject))]
-#![doc(html_root_url = "https://docs.rs/serde_with/3.4.0/")]
+#![doc(html_root_url = "https://docs.rs/serde_with/3.9.0/")]
 #![cfg_attr(docsrs, feature(doc_cfg))]
-#![allow(
-    // clippy is broken and shows wrong warnings
-    // clippy on stable does not know yet about the lint name
-    unknown_lints,
-    // https://github.com/rust-lang/rust-clippy/issues/8560
-    clippy::only_used_in_recursion,
-    // https://github.com/rust-lang/rust-clippy/issues/8867
-    clippy::derive_partial_eq_without_eq,
-    // https://github.com/rust-lang/rust-clippy/issues/9101
-    clippy::explicit_auto_deref
-)]
 #![no_std]
 
 //! [![crates.io badge](https://img.shields.io/crates/v/serde_with.svg)](https://crates.io/crates/serde_with/)
@@ -49,7 +33,7 @@
 //!
 //! ---
 //!
-//! This crate provides custom de/serialization helpers to use in combination with [serde's with-annotation][with-annotation] and with the improved [`serde_as`][as-annotation]-annotation.
+//! This crate provides custom de/serialization helpers to use in combination with [serde's `with` annotation][with-annotation] and with the improved [`serde_as`][as-annotation]-annotation.
 //! Some common use cases are:
 //!
 //! * De/Serializing a type using the `Display` and `FromStr` traits, e.g., for `u8`, `url::Url`, or `mime::Mime`.
@@ -93,11 +77,9 @@
 //!
 //! [![Rustexplorer](https://img.shields.io/badge/Try%20on-rustexplorer-lightgrey?logo=rust&logoColor=orange)](https://www.rustexplorer.com/b/py7ida)
 //! ```rust
-//! # #[cfg(feature = "macros")]
+//! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! # use serde::{Deserialize, Serialize};
-//! # #[cfg(feature = "macros")]
 //! # use serde_with::{serde_as, DisplayFromStr};
-//! # #[cfg(feature = "macros")]
 //! #[serde_as]
 //! # #[derive(Debug, Eq, PartialEq)]
 //! #[derive(Deserialize, Serialize)]
@@ -107,7 +89,6 @@
 //!     bar: u8,
 //! }
 //!
-//! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! // This will serialize
 //! # let foo =
 //! Foo {bar: 12}
@@ -118,7 +99,7 @@
 //! {"bar": "12"}
 //! # "#;
 //! # assert_eq!(json.replace(" ", "").replace("\n", ""), serde_json::to_string(&foo).unwrap());
-//! # assert_eq!(foo, serde_json::from_str(&json).unwrap());
+//! # assert_eq!(foo, serde_json::from_str(json).unwrap());
 //! # }
 //! ```
 //!
@@ -131,11 +112,9 @@
 //!
 //! [![Rustexplorer](https://img.shields.io/badge/Try%20on-rustexplorer-lightgrey?logo=rust&logoColor=orange)](https://www.rustexplorer.com/b/um0xyi)
 //! ```rust
-//! # #[cfg(feature = "macros")]
+//! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! # use serde::{Deserialize, Serialize};
-//! # #[cfg(feature = "macros")]
 //! # use serde_with::{serde_as, Bytes};
-//! # #[cfg(feature = "macros")]
 //! #[serde_as]
 //! # #[derive(Debug, Eq, PartialEq)]
 //! #[derive(Deserialize, Serialize)]
@@ -153,7 +132,6 @@
 //!     bytes: [u8; M],
 //! }
 //!
-//! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! // This allows us to serialize a struct like this
 //! let arrays: Arrays<100, 128> = Arrays {
 //!     constgeneric: [true; 100],
@@ -173,11 +151,9 @@
 //!
 //! [![Rustexplorer](https://img.shields.io/badge/Try%20on-rustexplorer-lightgrey?logo=rust&logoColor=orange)](https://www.rustexplorer.com/b/xr1tm0)
 //! ```rust
-//! # #[cfg(feature = "macros")]
+//! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! # use serde::{Deserialize, Serialize};
-//! # #[cfg(feature = "macros")]
 //! # use serde_with::skip_serializing_none;
-//! # #[cfg(feature = "macros")]
 //! #[skip_serializing_none]
 //! # #[derive(Debug, Eq, PartialEq)]
 //! #[derive(Deserialize, Serialize)]
@@ -191,7 +167,6 @@
 //!     g: Option<usize>,
 //! }
 //!
-//! # #[cfg(all(feature = "macros", feature = "json"))] {
 //! // This will serialize
 //! # let foo =
 //! Foo {a: None, b: None, c: None, d: Some(4), e: None, f: None, g: Some(7)}
@@ -202,13 +177,13 @@
 //! {"d": 4, "g": 7}
 //! # "#;
 //! # assert_eq!(json.replace(" ", "").replace("\n", ""), serde_json::to_string(&foo).unwrap());
-//! # assert_eq!(foo, serde_json::from_str(&json).unwrap());
+//! # assert_eq!(foo, serde_json::from_str(json).unwrap());
 //! # }
 //! ```
 //!
 //! ## Advanced `serde_as` usage
 //!
-//! This example is mainly supposed to highlight the flexibility of the `serde_as`-annotation compared to [serde's with-annotation][with-annotation].
+//! This example is mainly supposed to highlight the flexibility of the `serde_as` annotation compared to [serde's `with` annotation][with-annotation].
 //! More details about `serde_as` can be found in the [user guide].
 //!
 //! ```rust
@@ -217,6 +192,7 @@
 //! #     serde::{Deserialize, Serialize},
 //! #     serde_with::{serde_as, DisplayFromStr, DurationSeconds, hex::Hex, Map},
 //! # };
+//! # #[cfg(all(feature = "macros", feature = "hex"))]
 //! use std::time::Duration;
 //!
 //! # #[cfg(all(feature = "macros", feature = "hex"))]
@@ -252,7 +228,7 @@
 //! }
 //! # "#;
 //! # assert_eq!(json.replace(" ", "").replace("\n", ""), serde_json::to_string(&foo).unwrap());
-//! # assert_eq!(foo, serde_json::from_str(&json).unwrap());
+//! # assert_eq!(foo, serde_json::from_str(json).unwrap());
 //!
 //! // and serializes
 //! # let foo =
@@ -277,18 +253,18 @@
 //! }
 //! # "#;
 //! # assert_eq!(json.replace(" ", "").replace("\n", ""), serde_json::to_string(&foo).unwrap());
-//! # assert_eq!(foo, serde_json::from_str(&json).unwrap());
+//! # assert_eq!(foo, serde_json::from_str(json).unwrap());
 //! # }
 //! ```
 //!
-//! [`DisplayFromStr`]: https://docs.rs/serde_with/3.4.0/serde_with/struct.DisplayFromStr.html
-//! [`with_prefix!`]: https://docs.rs/serde_with/3.4.0/serde_with/macro.with_prefix.html
-//! [feature flags]: https://docs.rs/serde_with/3.4.0/serde_with/guide/feature_flags/index.html
-//! [skip_serializing_none]: https://docs.rs/serde_with/3.4.0/serde_with/attr.skip_serializing_none.html
-//! [StringWithSeparator]: https://docs.rs/serde_with/3.4.0/serde_with/struct.StringWithSeparator.html
-//! [user guide]: https://docs.rs/serde_with/3.4.0/serde_with/guide/index.html
+//! [`DisplayFromStr`]: https://docs.rs/serde_with/3.9.0/serde_with/struct.DisplayFromStr.html
+//! [`with_prefix!`]: https://docs.rs/serde_with/3.9.0/serde_with/macro.with_prefix.html
+//! [feature flags]: https://docs.rs/serde_with/3.9.0/serde_with/guide/feature_flags/index.html
+//! [skip_serializing_none]: https://docs.rs/serde_with/3.9.0/serde_with/attr.skip_serializing_none.html
+//! [StringWithSeparator]: https://docs.rs/serde_with/3.9.0/serde_with/struct.StringWithSeparator.html
+//! [user guide]: https://docs.rs/serde_with/3.9.0/serde_with/guide/index.html
 //! [with-annotation]: https://serde.rs/field-attrs.html#with
-//! [as-annotation]: https://docs.rs/serde_with/3.4.0/serde_with/guide/serde_as/index.html
+//! [as-annotation]: https://docs.rs/serde_with/3.9.0/serde_with/guide/serde_as/index.html
 
 #[cfg(feature = "alloc")]
 extern crate alloc;
@@ -296,6 +272,8 @@ extern crate alloc;
 pub extern crate core;
 #[doc(hidden)]
 pub extern crate serde;
+#[doc(hidden)]
+pub extern crate serde_derive;
 #[cfg(feature = "std")]
 extern crate std;
 
@@ -330,6 +308,9 @@ pub mod json;
 #[cfg(feature = "alloc")]
 mod key_value_map;
 pub mod rust;
+#[cfg(feature = "schemars_0_8")]
+#[cfg_attr(docsrs, doc(cfg(feature = "schemars_0_8")))]
+pub mod schemars_0_8;
 pub mod ser;
 #[cfg(feature = "std")]
 mod serde_conv;
@@ -405,6 +386,7 @@ pub(crate) mod prelude {
         marker::PhantomData,
         ops::Bound,
         option::Option,
+        pin::Pin,
         result::Result,
         str::FromStr,
         time::Duration,
@@ -455,8 +437,8 @@ pub use serde_with_macros::*;
 
 /// Adapter to convert from `serde_as` to the serde traits.
 ///
-/// The `As` type adapter allows using types which implement [`DeserializeAs`] or [`SerializeAs`] in place of serde's with-annotation.
-/// The with-annotation allows running custom code when de/serializing, however it is quite inflexible.
+/// The `As` type adapter allows using types which implement [`DeserializeAs`] or [`SerializeAs`] in place of serde's `with` annotation.
+/// The `with` annotation allows running custom code when de/serializing, however it is quite inflexible.
 /// The traits [`DeserializeAs`]/[`SerializeAs`] are more flexible, as they allow composition and nesting of types to create more complex de/serialization behavior.
 /// However, they are not directly compatible with serde, as they are not provided by serde.
 /// The `As` type adapter makes them compatible, by forwarding the function calls to `serialize`/`deserialize` to the corresponding functions `serialize_as` and `deserialize_as`.
@@ -470,6 +452,7 @@ pub use serde_with_macros::*;
 /// # use serde::{Deserialize, Serialize};
 /// # use serde_with::{As, DisplayFromStr};
 /// #
+/// # #[allow(dead_code)]
 /// #[derive(Deserialize, Serialize)]
 /// # struct S {
 /// // Serialize numbers as sequence of strings, using Display and FromStr
@@ -487,6 +470,7 @@ pub use serde_with_macros::*;
 /// # use serde_with::{As, DisplayFromStr, Same};
 /// # use std::collections::BTreeMap;
 /// #
+/// # #[allow(dead_code)]
 /// #[derive(Deserialize, Serialize)]
 /// # struct S {
 /// // Serialize map, turn keys into strings but keep type of value
@@ -496,13 +480,13 @@ pub use serde_with_macros::*;
 /// # }
 /// ```
 ///
-/// [serde_as]: https://docs.rs/serde_with/3.4.0/serde_with/attr.serde_as.html
+/// [serde_as]: https://docs.rs/serde_with/3.9.0/serde_with/attr.serde_as.html
 pub struct As<T: ?Sized>(PhantomData<T>);
 
 /// Adapter to convert from `serde_as` to the serde traits.
 ///
 /// This is the counter-type to [`As`][].
-/// It can be used whenever a type implementing [`DeserializeAs`][]/[`SerializeAs`][] is required but the normal `Deserialize`/`Serialize` traits should be used.
+/// It can be used whenever a type implementing [`DeserializeAs`]/[`SerializeAs`] is required but the normal [`Deserialize`](::serde::Deserialize)/[`Serialize`](::serde::Serialize) traits should be used.
 /// Check [`As`] for an example.
 pub struct Same;
 
@@ -546,7 +530,7 @@ pub struct Same;
 ///     mime: mime::STAR_STAR,
 ///     number: 777,
 /// };
-/// assert_eq!(json!({ "mime": "*/*", "number": "777" }), serde_json::to_value(&x).unwrap());
+/// assert_eq!(json!({ "mime": "*/*", "number": "777" }), serde_json::to_value(x).unwrap());
 /// # }
 /// ```
 ///
@@ -554,9 +538,52 @@ pub struct Same;
 /// [`FromStr`]: std::str::FromStr
 pub struct DisplayFromStr;
 
+/// Use the first format if [`De/Serializer::is_human_readable`], otherwise use the second
+///
+/// If the second format is not specified, the normal
+/// [`Deserialize`](::serde::Deserialize)/[`Serialize`](::serde::Serialize) traits are used.
+///
+/// # Examples
+///
+/// ```rust
+/// # #[cfg(feature = "macros")] {
+/// # use serde::{Deserialize, Serialize};
+/// # use serde_json::json;
+/// # use serde_with::{serde_as, DisplayFromStr, IfIsHumanReadable, DurationMilliSeconds, DurationSeconds};
+/// use std::time::Duration;
+///
+/// #[serde_as]
+/// #[derive(Deserialize, Serialize)]
+/// struct A {
+///     #[serde_as(as = "IfIsHumanReadable<DisplayFromStr>")]
+///     number: u32,
+/// }
+/// let x = A {
+///     number: 777,
+/// };
+/// assert_eq!(json!({ "number": "777" }), serde_json::to_value(&x).unwrap());
+/// assert_eq!(vec![145, 205, 3, 9], rmp_serde::to_vec(&x).unwrap());
+///
+/// #[serde_as]
+/// #[derive(Deserialize, Serialize)]
+/// struct B {
+///     #[serde_as(as = "IfIsHumanReadable<DurationMilliSeconds, DurationSeconds>")]
+///     duration: Duration,
+/// }
+/// let x = B {
+///     duration: Duration::from_millis(1500),
+/// };
+/// assert_eq!(json!({ "duration": 1500 }), serde_json::to_value(&x).unwrap());
+/// assert_eq!(vec![145, 2], rmp_serde::to_vec(&x).unwrap());
+/// # }
+/// ```
+/// [`De/Serializer::is_human_readable`]: serde::Serializer::is_human_readable
+/// [`is_human_readable`]: serde::Serializer::is_human_readable
+pub struct IfIsHumanReadable<H, F = Same>(PhantomData<H>, PhantomData<F>);
+
 /// De/Serialize a [`Option<String>`] type while transforming the empty string to [`None`]
 ///
-/// Convert an [`Option<T>`] from/to string using [`FromStr`] and [`Display`] implementations.
+/// Convert an [`Option<T>`] from/to string using [`FromStr`] and [`Display`](::core::fmt::Display) implementations.
 /// An empty string is deserialized as [`None`] and a [`None`] vice versa.
 ///
 /// # Examples
@@ -583,12 +610,12 @@ pub struct DisplayFromStr;
 /// let x = A {
 ///     tags: Some("This is text".to_string()),
 /// };
-/// assert_eq!(json!({ "tags": "This is text" }), serde_json::to_value(&x).unwrap());
+/// assert_eq!(json!({ "tags": "This is text" }), serde_json::to_value(x).unwrap());
 ///
 /// let x = A {
 ///     tags: None,
 /// };
-/// assert_eq!(json!({ "tags": "" }), serde_json::to_value(&x).unwrap());
+/// assert_eq!(json!({ "tags": "" }), serde_json::to_value(x).unwrap());
 /// # }
 /// ```
 ///
@@ -673,7 +700,7 @@ pub struct NoneAsEmptyString;
 /// struct C {
 ///     #[serde_as(as = "Vec<DefaultOnError<DisplayFromStr>>")]
 ///     value: Vec<u32>,
-/// };
+/// }
 ///
 /// let c: C = serde_json::from_value(json!({
 ///     "value": ["1", "2", "a3", "", {}, "6"]
@@ -729,7 +756,7 @@ pub struct DefaultOnError<T = Same>(PhantomData<T>);
 /// struct C {
 ///     #[serde_as(as = "Vec<DefaultOnNull<DisplayFromStr>>")]
 ///     value: Vec<u32>,
-/// };
+/// }
 ///
 /// let c: C = serde_json::from_value(json!({
 ///     "value": ["1", "2", null, null, "5"]
@@ -790,8 +817,8 @@ pub struct BytesOrString;
 
 /// De/Serialize Durations as number of seconds.
 ///
-/// De/serialize durations as number of seconds with subsecond precision.
-/// Subsecond precision is *only* supported for [`DurationSecondsWithFrac`], but not for [`DurationSeconds`].
+/// De/serialize durations as number of seconds with sub-second precision.
+/// Sub-second precision is *only* supported for [`DurationSecondsWithFrac`], but not for [`DurationSeconds`].
 /// You can configure the serialization format between integers, floats, and stringified numbers with the `FORMAT` specifier and configure the deserialization with the `STRICTNESS` specifier.
 ///
 /// The `STRICTNESS` specifier can either be [`formats::Strict`] or [`formats::Flexible`] and defaults to [`formats::Strict`].
@@ -834,7 +861,7 @@ pub struct BytesOrString;
 ///     d_f64: Duration,
 ///     #[serde_as(as = "DurationSeconds<String>")]
 ///     d_string: Duration,
-/// };
+/// }
 ///
 /// // Serialization
 /// // See how the values get rounded, since subsecond precision is not allowed.
@@ -850,7 +877,7 @@ pub struct BytesOrString;
 ///     "d_f64": 12346.0,
 ///     "d_string": "12346",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&d).unwrap());
+/// assert_eq!(expected, serde_json::to_value(d).unwrap());
 ///
 /// // Deserialization works too
 /// // Subsecond precision in numbers will be rounded away
@@ -892,7 +919,7 @@ pub struct BytesOrString;
 ///     d_f64: Duration,
 ///     #[serde_as(as = "DurationSeconds<String>")]
 ///     d_string: Duration,
-/// };
+/// }
 ///
 /// // Serialization
 /// // See how the values get rounded, since subsecond precision is not allowed.
@@ -908,7 +935,7 @@ pub struct BytesOrString;
 ///     "d_f64": -12345.0,
 ///     "d_string": "12346",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&d).unwrap());
+/// assert_eq!(expected, serde_json::to_value(d).unwrap());
 ///
 /// // Deserialization works too
 /// // Subsecond precision in numbers will be rounded away
@@ -928,7 +955,7 @@ pub struct BytesOrString;
 /// ```
 ///
 /// [`chrono::Duration`]: ::chrono_0_4::Duration
-/// [feature flag]: https://docs.rs/serde_with/3.4.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.9.0/serde_with/guide/feature_flags/index.html
 pub struct DurationSeconds<
     FORMAT: formats::Format = u64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -978,7 +1005,7 @@ pub struct DurationSeconds<
 ///     d_f64: Duration,
 ///     #[serde_as(as = "DurationSecondsWithFrac<String>")]
 ///     d_string: Duration,
-/// };
+/// }
 ///
 /// // Serialization
 /// // See how the values get rounded, since subsecond precision is not allowed.
@@ -992,7 +1019,7 @@ pub struct DurationSeconds<
 ///     "d_f64": 12345.5,
 ///     "d_string": "12345.999999",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&d).unwrap());
+/// assert_eq!(expected, serde_json::to_value(d).unwrap());
 ///
 /// // Deserialization works too
 /// // Subsecond precision in numbers will be rounded away
@@ -1030,7 +1057,7 @@ pub struct DurationSeconds<
 ///     d_f64: Duration,
 ///     #[serde_as(as = "DurationSecondsWithFrac<String>")]
 ///     d_string: Duration,
-/// };
+/// }
 ///
 /// // Serialization
 ///
@@ -1043,7 +1070,7 @@ pub struct DurationSeconds<
 ///     "d_f64": -12344.5,
 ///     "d_string": "12345.999999",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&d).unwrap());
+/// assert_eq!(expected, serde_json::to_value(d).unwrap());
 ///
 /// // Deserialization works too
 ///
@@ -1060,7 +1087,7 @@ pub struct DurationSeconds<
 /// ```
 ///
 /// [`chrono::Duration`]: ::chrono_0_4::Duration
-/// [feature flag]: https://docs.rs/serde_with/3.4.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.9.0/serde_with/guide/feature_flags/index.html
 pub struct DurationSecondsWithFrac<
     FORMAT: formats::Format = f64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1166,7 +1193,7 @@ pub struct DurationNanoSecondsWithFrac<
 ///     st_f64: SystemTime,
 ///     #[serde_as(as = "TimestampSeconds<String>")]
 ///     st_string: SystemTime,
-/// };
+/// }
 ///
 /// // Serialization
 /// // See how the values get rounded, since subsecond precision is not allowed.
@@ -1182,7 +1209,7 @@ pub struct DurationNanoSecondsWithFrac<
 ///     "st_f64": 12346.0,
 ///     "st_string": "12346",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&ts).unwrap());
+/// assert_eq!(expected, serde_json::to_value(ts).unwrap());
 ///
 /// // Deserialization works too
 /// // Subsecond precision in numbers will be rounded away
@@ -1224,15 +1251,15 @@ pub struct DurationNanoSecondsWithFrac<
 ///     dt_f64: DateTime<Local>,
 ///     #[serde_as(as = "TimestampSeconds<String>")]
 ///     dt_string: DateTime<Utc>,
-/// };
+/// }
 ///
 /// // Serialization
 /// // See how the values get rounded, since subsecond precision is not allowed.
 ///
 /// let ts = Timestamps {
-///     dt_i64: Utc.timestamp(-12345, 0),
-///     dt_f64: Local.timestamp(-12345, 500_000_000),
-///     dt_string: Utc.timestamp(12345, 999_999_999),
+///     dt_i64: Utc.timestamp_opt(-12345, 0).unwrap(),
+///     dt_f64: Local.timestamp_opt(-12345, 500_000_000).unwrap(),
+///     dt_string: Utc.timestamp_opt(12345, 999_999_999).unwrap(),
 /// };
 /// // Observe the different data types
 /// let expected = json!({
@@ -1240,7 +1267,7 @@ pub struct DurationNanoSecondsWithFrac<
 ///     "dt_f64": -12345.0,
 ///     "dt_string": "12346",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&ts).unwrap());
+/// assert_eq!(expected, serde_json::to_value(ts).unwrap());
 ///
 /// // Deserialization works too
 /// // Subsecond precision in numbers will be rounded away
@@ -1251,9 +1278,9 @@ pub struct DurationNanoSecondsWithFrac<
 ///     "dt_string": "12346",
 /// });
 /// let expected = Timestamps {
-///     dt_i64: Utc.timestamp(-12345, 0),
-///     dt_f64: Local.timestamp(-12346, 0),
-///     dt_string: Utc.timestamp(12346, 0),
+///     dt_i64: Utc.timestamp_opt(-12345, 0).unwrap(),
+///     dt_f64: Local.timestamp_opt(-12346, 0).unwrap(),
+///     dt_string: Utc.timestamp_opt(12346, 0).unwrap(),
 /// };
 /// assert_eq!(expected, serde_json::from_value(json).unwrap());
 /// # }
@@ -1262,7 +1289,7 @@ pub struct DurationNanoSecondsWithFrac<
 /// [`SystemTime`]: std::time::SystemTime
 /// [`chrono::DateTime<Local>`]: ::chrono_0_4::DateTime
 /// [`chrono::DateTime<Utc>`]: ::chrono_0_4::DateTime
-/// [feature flag]: https://docs.rs/serde_with/3.4.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.9.0/serde_with/guide/feature_flags/index.html
 pub struct TimestampSeconds<
     FORMAT: formats::Format = i64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1318,7 +1345,7 @@ pub struct TimestampSeconds<
 ///     st_f64: SystemTime,
 ///     #[serde_as(as = "TimestampSecondsWithFrac<String>")]
 ///     st_string: SystemTime,
-/// };
+/// }
 ///
 /// // Serialization
 /// // See how the values get rounded, since subsecond precision is not allowed.
@@ -1332,7 +1359,7 @@ pub struct TimestampSeconds<
 ///     "st_f64": 12345.5,
 ///     "st_string": "12345.999999",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&ts).unwrap());
+/// assert_eq!(expected, serde_json::to_value(ts).unwrap());
 ///
 /// // Deserialization works too
 /// // Subsecond precision in numbers will be rounded away
@@ -1370,20 +1397,20 @@ pub struct TimestampSeconds<
 ///     dt_f64: DateTime<Utc>,
 ///     #[serde_as(as = "TimestampSecondsWithFrac<String>")]
 ///     dt_string: DateTime<Local>,
-/// };
+/// }
 ///
 /// // Serialization
 ///
 /// let ts = Timestamps {
-///     dt_f64: Utc.timestamp(-12345, 500_000_000),
-///     dt_string: Local.timestamp(12345, 999_999_000),
+///     dt_f64: Utc.timestamp_opt(-12345, 500_000_000).unwrap(),
+///     dt_string: Local.timestamp_opt(12345, 999_999_000).unwrap(),
 /// };
 /// // Observe the different data types
 /// let expected = json!({
 ///     "dt_f64": -12344.5,
 ///     "dt_string": "12345.999999",
 /// });
-/// assert_eq!(expected, serde_json::to_value(&ts).unwrap());
+/// assert_eq!(expected, serde_json::to_value(ts).unwrap());
 ///
 /// // Deserialization works too
 ///
@@ -1392,8 +1419,8 @@ pub struct TimestampSeconds<
 ///     "dt_string": "12345.987",
 /// });
 /// let expected = Timestamps {
-///     dt_f64: Utc.timestamp(-12345, 500_000_000),
-///     dt_string: Local.timestamp(12345, 987_000_000),
+///     dt_f64: Utc.timestamp_opt(-12345, 500_000_000).unwrap(),
+///     dt_string: Local.timestamp_opt(12345, 987_000_000).unwrap(),
 /// };
 /// assert_eq!(expected, serde_json::from_value(json).unwrap());
 /// # }
@@ -1404,7 +1431,7 @@ pub struct TimestampSeconds<
 /// [`chrono::DateTime<Local>`]: ::chrono_0_4::DateTime
 /// [`chrono::DateTime<Utc>`]: ::chrono_0_4::DateTime
 /// [NaiveDateTime]: ::chrono_0_4::NaiveDateTime
-/// [feature flag]: https://docs.rs/serde_with/3.4.0/serde_with/guide/feature_flags/index.html
+/// [feature flag]: https://docs.rs/serde_with/3.9.0/serde_with/guide/feature_flags/index.html
 pub struct TimestampSecondsWithFrac<
     FORMAT: formats::Format = f64,
     STRICTNESS: formats::Strictness = formats::Strict,
@@ -1463,7 +1490,7 @@ pub struct TimestampNanoSecondsWithFrac<
 /// Serialization of byte sequences like `&[u8]` or `Vec<u8>` is quite inefficient since each value will be serialized individually.
 /// This converter type optimizes the serialization and deserialization.
 ///
-/// This is a port of the [`serde_bytes`] crate making it compatible with the `serde_as`-annotation, which allows it to be used in more cases than provided by [`serde_bytes`].
+/// This is a port of the [`serde_bytes`] crate making it compatible with the `serde_as` annotation, which allows it to be used in more cases than provided by [`serde_bytes`].
 ///
 /// The type provides de/serialization for these types:
 ///
@@ -1490,7 +1517,6 @@ pub struct TimestampNanoSecondsWithFrac<
 /// # #[derive(Debug, PartialEq)]
 /// #[derive(Deserialize, Serialize)]
 /// struct Test<'a> {
-/// #   #[cfg(FALSE)]
 ///     #[serde_as(as = "Bytes")]
 ///     array: [u8; 15],
 ///     #[serde_as(as = "Bytes")]
@@ -1498,7 +1524,6 @@ pub struct TimestampNanoSecondsWithFrac<
 ///     #[serde_as(as = "Bytes")]
 ///     #[serde(borrow)]
 ///     cow: Cow<'a, [u8]>,
-/// #   #[cfg(FALSE)]
 ///     #[serde_as(as = "Bytes")]
 ///     #[serde(borrow)]
 ///     cow_array: Cow<'a, [u8; 15]>,
@@ -1507,11 +1532,9 @@ pub struct TimestampNanoSecondsWithFrac<
 /// }
 ///
 /// let value = Test {
-/// #   #[cfg(FALSE)]
-///     array: b"0123456789ABCDE".clone(),
+///     array: *b"0123456789ABCDE",
 ///     boxed: b"...".to_vec().into_boxed_slice(),
 ///     cow: Cow::Borrowed(b"FooBar"),
-/// #   #[cfg(FALSE)]
 ///     cow_array: Cow::Borrowed(&[42u8; 15]),
 ///     vec: vec![0x41, 0x61, 0x21],
 /// };
@@ -1522,18 +1545,11 @@ pub struct TimestampNanoSecondsWithFrac<
 ///     cow_array: "KioqKioqKioqKioqKioq",
 ///     vec: "QWEh",
 /// )"#;
-/// # drop(expected);
-/// # // Create a fake expected value that doesn't use const generics
-/// # let expected = r#"(
-/// #     boxed: "Li4u",
-/// #     cow: "Rm9vQmFy",
-/// #     vec: "QWEh",
-/// # )"#;
 ///
 /// # let pretty_config = ron::ser::PrettyConfig::new()
 /// #     .new_line("\n".into());
 /// assert_eq!(expected, ron::ser::to_string_pretty(&value, pretty_config).unwrap());
-/// assert_eq!(value, ron::from_str(&expected).unwrap());
+/// assert_eq!(value, ron::from_str(expected).unwrap());
 /// # }
 /// ```
 ///
@@ -1544,13 +1560,11 @@ pub struct TimestampNanoSecondsWithFrac<
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
 /// # use serde_with::{serde_as, Bytes};
-/// # use std::borrow::Cow;
 /// #
 /// #[serde_as]
 /// # #[derive(Debug, PartialEq)]
 /// #[derive(Deserialize, Serialize)]
 /// struct TestBorrows<'a> {
-/// #   #[cfg(FALSE)]
 ///     #[serde_as(as = "Bytes")]
 ///     #[serde(borrow)]
 ///     array_buf: &'a [u8; 15],
@@ -1560,7 +1574,6 @@ pub struct TimestampNanoSecondsWithFrac<
 /// }
 ///
 /// let value = TestBorrows {
-/// #   #[cfg(FALSE)]
 ///     array_buf: &[10u8; 15],
 ///     buf: &[20u8, 21u8, 22u8],
 /// };
@@ -1568,11 +1581,6 @@ pub struct TimestampNanoSecondsWithFrac<
 ///     array_buf: "CgoKCgoKCgoKCgoKCgoK",
 ///     buf: "FBUW",
 /// )"#;
-/// # drop(expected);
-/// # // Create a fake expected value that doesn't use const generics
-/// # let expected = r#"(
-/// #     buf: "FBUW",
-/// # )"#;
 ///
 /// # let pretty_config = ron::ser::PrettyConfig::new()
 /// #     .new_line("\n".into());
@@ -1634,8 +1642,11 @@ pub struct Bytes;
 /// This distinction is not semantically important on the Rust side, thus both forms should deserialize into the same `Vec`.
 ///
 /// The `OneOrMany` adapter achieves exactly this use case.
-/// The serialization behavior can be tweaked to either always serialize as a list using `PreferMany` or to serialize as the inner element if possible using `PreferOne`.
-/// By default, `PreferOne` is assumed, which can also be omitted like `OneOrMany<_>`.
+/// The serialization behavior can be tweaked to either always serialize as a list using [`PreferMany`] or to serialize as the inner element if possible using [`PreferOne`].
+/// By default, [`PreferOne`] is assumed, which can also be omitted like `OneOrMany<_>`.
+///
+/// [`PreferMany`]: crate::formats::PreferMany
+/// [`PreferOne`]: crate::formats::PreferOne
 ///
 /// # Examples
 ///
@@ -1650,9 +1661,9 @@ pub struct Bytes;
 /// # #[derive(Debug, PartialEq)]
 /// #[derive(Deserialize, serde::Serialize)]
 /// struct Data {
-///     #[serde_as(deserialize_as = "OneOrMany<_, PreferOne>")]
+///     #[serde_as(as = "OneOrMany<_, PreferOne>")]
 ///     countries: Vec<String>,
-///     #[serde_as(deserialize_as = "OneOrMany<_, PreferMany>")]
+///     #[serde_as(as = "OneOrMany<_, PreferMany>")]
 ///     cities: Vec<String>,
 /// }
 ///
@@ -1681,7 +1692,7 @@ pub struct Bytes;
 ///     "countries": "Spain",
 ///     "cities": ["Berlin"],
 /// });
-/// assert_eq!(data, serde_json::from_value(j).unwrap());
+/// assert_eq!(serde_json::to_value(data).unwrap(), j);
 /// # }
 /// ```
 #[cfg(feature = "alloc")]
@@ -1754,7 +1765,7 @@ pub struct PickFirst<T>(PhantomData<T>);
 /// Deserializing works analogue, by deserializing a `T` and then converting into `O`.
 ///
 /// ```rust
-/// # #[cfg(FALSE)] {
+/// # #[cfg(any())] {
 /// struct S {
 ///     #[serde_as(as = "FromInto<T>")]
 ///     value: O,
@@ -1766,7 +1777,7 @@ pub struct PickFirst<T>(PhantomData<T>);
 /// For deserialization the opposite `T: Into<O>` is required.
 /// The `Clone` bound is required since `serialize` operates on a reference but `Into` implementations on references are uncommon.
 ///
-/// **Note**: [`TryFromInto`] is the more generalized version of this adapter which uses the [`TryInto`](std::convert::TryInto) trait instead.
+/// **Note**: [`TryFromInto`] is the more generalized version of this adapter which uses the [`TryInto`] trait instead.
 ///
 /// # Example
 ///
@@ -1836,7 +1847,7 @@ pub struct FromInto<T>(PhantomData<T>);
 /// Deserializing works analogue, by deserializing a `T` and then converting into `O`.
 ///
 /// ```rust
-/// # #[cfg(FALSE)] {
+/// # #[cfg(any())] {
 /// struct S {
 ///     #[serde_as(as = "FromIntoRef<T>")]
 ///     value: O,
@@ -1847,7 +1858,7 @@ pub struct FromInto<T>(PhantomData<T>);
 /// For serialization `O` needs to be `for<'a> &'a O: Into<T>`.
 /// For deserialization the opposite `T: Into<O>` is required.
 ///
-/// **Note**: [`TryFromIntoRef`] is the more generalized version of this adapter which uses the [`TryInto`](std::convert::TryInto) trait instead.
+/// **Note**: [`TryFromIntoRef`] is the more generalized version of this adapter which uses the [`TryInto`] trait instead.
 ///
 /// # Example
 ///
@@ -1917,7 +1928,7 @@ pub struct FromIntoRef<T>(PhantomData<T>);
 /// Deserializing works analogue, by deserializing a `T` and then converting into `O`.
 ///
 /// ```rust
-/// # #[cfg(FALSE)] {
+/// # #[cfg(any())] {
 /// struct S {
 ///     #[serde_as(as = "TryFromInto<T>")]
 ///     value: O,
@@ -1941,7 +1952,6 @@ pub struct FromIntoRef<T>(PhantomData<T>);
 /// # use serde::{Deserialize, Serialize};
 /// # use serde_json::json;
 /// # use serde_with::{serde_as, TryFromInto};
-/// # use std::convert::TryFrom;
 /// #
 /// #[derive(Clone, Debug, PartialEq)]
 /// enum Boollike {
@@ -2007,7 +2017,7 @@ pub struct TryFromInto<T>(PhantomData<T>);
 /// Deserializing works analogue, by deserializing a `T` and then converting into `O`.
 ///
 /// ```rust
-/// # #[cfg(FALSE)] {
+/// # #[cfg(any())] {
 /// struct S {
 ///     #[serde_as(as = "TryFromIntoRef<T>")]
 ///     value: O,
@@ -2030,7 +2040,6 @@ pub struct TryFromInto<T>(PhantomData<T>);
 /// # use serde::{Deserialize, Serialize};
 /// # use serde_json::json;
 /// # use serde_with::{serde_as, TryFromIntoRef};
-/// # use std::convert::TryFrom;
 /// #
 /// #[derive(Debug, PartialEq)]
 /// enum Boollike {
@@ -2196,6 +2205,64 @@ pub struct BorrowCow;
 #[cfg(feature = "alloc")]
 pub struct VecSkipError<T>(PhantomData<T>);
 
+/// Deserialize a map, skipping keys and values which fail to deserialize.
+///
+/// By default serde terminates if it fails to deserialize a key or a value when deserializing
+/// a map. Sometimes a map has heterogeneous keys or values but we only care about some specific
+/// types, and it is desirable to skip entries on errors.
+///
+/// It is especially useful in conjunction to `#[serde(flatten)]` to capture a map mixed in with
+/// other entries which we don't want to exhaust in the type definition.
+///
+/// The serialization behavior is identical to the underlying map.
+///
+/// The implementation supports both the [`HashMap`] and the [`BTreeMap`] from the standard library.
+///
+/// [`BTreeMap`]: std::collections::BTreeMap
+/// [`HashMap`]: std::collections::HashMap
+///
+/// # Examples
+///
+/// ```rust
+/// # #[cfg(feature = "macros")] {
+/// # use serde::{Deserialize, Serialize};
+/// # use std::collections::BTreeMap;
+/// # use serde_with::{serde_as, DisplayFromStr, MapSkipError};
+/// #
+/// #[serde_as]
+/// # #[derive(Debug, PartialEq)]
+/// #[derive(Deserialize, Serialize)]
+/// struct VersionNames {
+///     yanked: Vec<u16>,
+///     #[serde_as(as = "MapSkipError<DisplayFromStr, _>")]
+///     #[serde(flatten)]
+///     names: BTreeMap<u16, String>,
+/// }
+///
+/// let data = VersionNames {
+///     yanked: vec![2, 5],
+///     names: BTreeMap::from_iter([
+///         (0u16, "v0".to_string()),
+///         (1, "v1".to_string()),
+///         (4, "v4".to_string())
+///     ]),
+/// };
+/// let source_json = r#"{
+///   "0": "v0",
+///   "1": "v1",
+///   "4": "v4",
+///   "yanked": [2, 5],
+///   "last_updated": 1704085200
+/// }"#;
+/// let data_json = r#"{"yanked":[2,5],"0":"v0","1":"v1","4":"v4"}"#;
+/// // Ensure serialization and deserialization produce the expected results
+/// assert_eq!(data_json, serde_json::to_string(&data).unwrap());
+/// assert_eq!(data, serde_json::from_str(source_json).unwrap());
+/// # }
+/// ```
+#[cfg(feature = "alloc")]
+pub struct MapSkipError<K, V>(PhantomData<(K, V)>);
+
 /// Deserialize a boolean from a number
 ///
 /// Deserialize a number (of `u8`) and turn it into a boolean.
@@ -2313,7 +2380,6 @@ pub struct StringWithSeparator<Sep, T>(PhantomData<(Sep, T)>);
 /// ```rust
 /// # #[cfg(feature = "macros")] {
 /// # use serde::{Deserialize, Serialize};
-/// # use serde_json::json;
 /// # use serde_with::{serde_as, Map};
 /// #
 /// #[serde_as]
@@ -2406,8 +2472,8 @@ pub struct Seq<V>(PhantomData<V>);
 ///
 /// The implementation supports both the [`HashMap`] and the [`BTreeMap`] from the standard library.
 ///
+/// [`BTreeMap`]: std::collections::BTreeMap
 /// [`HashMap`]: std::collections::HashMap
-/// [`BTreeMap`]: std::collections::HashMap
 ///
 /// # Example
 ///
@@ -2444,15 +2510,15 @@ pub struct Seq<V>(PhantomData<V>);
 #[cfg(feature = "alloc")]
 pub struct MapPreventDuplicates<K, V>(PhantomData<(K, V)>);
 
-/// Ensure that the last value is taken, if duplicate values exist
+/// Ensure that the first key is taken, if duplicate keys exist
 ///
-/// By default serde has a first-value-wins implementation, if duplicate keys for a set exist.
-/// Sometimes the opposite strategy is desired. This helper implements a first-value-wins strategy.
+/// By default serde has a last-key-wins implementation, if duplicate keys for a map exist.
+/// Sometimes the opposite strategy is desired. This helper implements a first-key-wins strategy.
 ///
-/// The implementation supports both the [`HashSet`] and the [`BTreeSet`] from the standard library.
+/// The implementation supports both the [`HashMap`] and the [`BTreeMap`] from the standard library.
 ///
-/// [`HashSet`]: std::collections::HashSet
-/// [`BTreeSet`]: std::collections::HashSet
+/// [`BTreeMap`]: std::collections::BTreeMap
+/// [`HashMap`]: std::collections::HashMap
 #[cfg(feature = "alloc")]
 pub struct MapFirstKeyWins<K, V>(PhantomData<(K, V)>);
 
@@ -2466,8 +2532,8 @@ pub struct MapFirstKeyWins<K, V>(PhantomData<(K, V)>);
 ///
 /// The implementation supports both the [`HashSet`] and the [`BTreeSet`] from the standard library.
 ///
+/// [`BTreeSet`]: std::collections::BTreeSet
 /// [`HashSet`]: std::collections::HashSet
-/// [`BTreeSet`]: std::collections::HashSet
 ///
 /// # Example
 ///
@@ -2508,7 +2574,17 @@ pub struct SetPreventDuplicates<T>(PhantomData<T>);
 ///
 /// The implementation supports both the [`HashSet`] and the [`BTreeSet`] from the standard library.
 ///
+/// [`BTreeSet`]: std::collections::BTreeSet
 /// [`HashSet`]: std::collections::HashSet
-/// [`BTreeSet`]: std::collections::HashSet
 #[cfg(feature = "alloc")]
 pub struct SetLastValueWins<T>(PhantomData<T>);
+
+/// Helper for implementing [`JsonSchema`] on serializers whose output depends
+/// on the type of the concrete field.
+///
+/// It is added implicitly by the [`#[serde_as]`](crate::serde_as) macro when any `schemars`
+/// feature is enabled.
+///
+/// [`JsonSchema`]: ::schemars_0_8::JsonSchema
+#[cfg(feature = "schemars_0_8")]
+pub struct Schema<T: ?Sized, TA>(PhantomData<T>, PhantomData<TA>);
